@@ -1,3 +1,4 @@
+import enum
 import json
 import logging
 import time
@@ -6,6 +7,8 @@ from urllib.parse import urlparse
 
 import paho.mqtt.client as mqtt
 import requests
+import typing
+
 from chattyraspi.device import DevicesConfiguration
 
 
@@ -15,54 +18,82 @@ class GraphqlException(Exception):
 
 # Subscriptions are not supported by Cloudfront. So we have to keep the original URL.
 _ROOT_URL = 'https://c7knkzejobbqpnaz4gskh77nmm.appsync-api.eu-west-1.amazonaws.com/graphql'
+_DEV_ROOT_URL = 'https://dnbcs5up6jcyro7ejt3xgc4zbu.appsync-api.eu-west-1.amazonaws.com/graphql'
 
 
-def _do_nothing(_):
+def _do_nothing(*_):
     pass
 
 
+@enum.unique
+class ThermostatMode(enum.Enum):
+    HEAT = 'HEAT'
+    COOL = 'COOL'
+    AUTO = 'AUTO'
+    OFF = 'OFF'
+
+
+DeviceID = typing.NewType('DeviceID', str)
+
+
 class DeviceIdClient:
-    def __init__(self, device_id: str, openid_token: str, reconnect_time_seconds: int):
+    # noinspection PyTypeChecker
+    def __init__(self, device_id: str, openid_token: str, reconnect_time_seconds: int, appsync_url: str):
         self._headers = {
             'Authorization': openid_token
         }
         self._device_id = device_id
-        self.on_turn_on = _do_nothing
-        self.on_turn_off = _do_nothing
-        self.fetch_is_power_on = _do_nothing
+        self.on_turn_on = _do_nothing  # type: typing.Callable[[DeviceID], None]
+        self.on_turn_off = _do_nothing  # type: typing.Callable[[DeviceID], None]
+        self.fetch_is_power_on = _do_nothing  # type: typing.Callable[[DeviceID], typing.Optional[bool]]
+        self.fetch_temperature = _do_nothing  # type: typing.Callable[[DeviceID], typing.Optional[float]]
+        self.fetch_thermostat_mode = _do_nothing  # type: typing.Callable[[DeviceID], typing.Optional[ThermostatMode]]
+        self.fetch_thermostat_target_setpoint = _do_nothing  # type: typing.Callable[[DeviceID], typing.Optional[float]]
+
+        self.on_set_temperature = _do_nothing  # type: typing.Callable[[DeviceID, float], None]
+        self.on_adjust_temperature = _do_nothing  # type: typing.Callable[[DeviceID, float], None]
+        self.on_set_thermostat_mode = _do_nothing  # type: typing.Callable[[DeviceID, ThermostatMode], None]
+
         self._logger = logging.getLogger('client.raspi.alexa.mirko.io')
         self._reconnect_time_seconds = reconnect_time_seconds
+        self._appsync_url = appsync_url
 
     @property
     def device_id(self) -> str:
         return self._device_id
 
     def _info(self, s: str, *args, **kwargs):
+        # noinspection PyArgumentList
         self._log(self._logger.info, s, *args, **kwargs)
 
     def _debug(self, s: str, *args, **kwargs):
+        # noinspection PyArgumentList
         self._log(self._logger.debug, s, *args, **kwargs)
 
     def _warning(self, s: str, *args, **kwargs):
+        # noinspection PyArgumentList
         self._log(self._logger.warning, s, *args, **kwargs)
 
     def _error(self, s: str, *args, **kwargs):
+        # noinspection PyArgumentList
         self._log(self._logger.error, s, *args, **kwargs)
 
     def _exception(self, s: str, *args, **kwargs):
+        # noinspection PyArgumentList
         self._log(self._logger.exception, s, *args, **kwargs)
 
-    def _log(self, log_function, s: str, *args, **kwargs):
+    def _log(self, log_function: callable, s: str, *args, **kwargs):
+        # noinspection PyArgumentList
         log_function('[%s] ' + s, self._device_id, *args, **kwargs)
 
     def _query(self, q: str):
-        response = requests.post(_ROOT_URL, json={'query': q}, headers=self._headers)
+        response = requests.post(self._appsync_url, json={'query': q}, headers=self._headers)
         response.raise_for_status()
         return response.json()
 
     def _mutation(self, m: str, v: dict):
         response = requests.post(
-            _ROOT_URL, json={'query': m, 'variables': v}, headers=self._headers)
+            self._appsync_url, json={'query': m, 'variables': v}, headers=self._headers)
         response.raise_for_status()
         response = response.json()
         if response.get('errors'):
@@ -95,7 +126,7 @@ class DeviceIdClient:
             "variables": {"deviceId": self._device_id}
         }
 
-        r = requests.post(_ROOT_URL, headers=post_headers, json=payload)
+        r = requests.post(self._appsync_url, headers=post_headers, json=payload)
         try:
             r.raise_for_status()
         except Exception:
@@ -127,18 +158,29 @@ class DeviceIdClient:
             command_payload = json.loads(msg.payload.decode())
             command = command_payload['data']['onCommandCreated']['command']
             command_id = command_payload['data']['onCommandCreated']['commandId']
+            arguments = command_payload['data']['onCommandCreated']['arguments']
 
-            response_execution = lambda: self._command_executed(command_id)
+            def response_execution():
+                self._command_executed(command_id)
 
             # noinspection PyBroadException
             try:
+                self._logger.info('Received command %s(%s)', command, arguments)
                 if command == 'turnOn':
                     self.on_turn_on(self._device_id)
                 elif command == 'turnOff':
                     self.on_turn_off(self._device_id)
                 elif command == 'powerStatus':
-                    response = 'ON' if self.fetch_is_power_on(self._device_id) else 'OFF'
-                    response_execution = lambda: self._command_responded(command_id, response)
+                    response_execution = self._on_power_status(command_id)
+                elif command == 'setTemperature':
+                    self.on_set_temperature(self._device_id, float(arguments[0]))
+                    response_execution = self._on_thermostat_changed(command_id)
+                elif command == 'adjustTemperature':
+                    self.on_adjust_temperature(self._device_id, float(arguments[0]))
+                    response_execution = self._on_thermostat_changed(command_id)
+                elif command == 'setThermostatMode':
+                    self.on_set_thermostat_mode(self._device_id, ThermostatMode(arguments[0]))
+                    response_execution = self._on_thermostat_changed(command_id)
                 else:
                     raise ValueError('Unexpected command {}, payload {}'.format(msg.payload['command'], msg))
                 response_execution()
@@ -180,6 +222,51 @@ class DeviceIdClient:
             client.disconnect()
             del client
 
+    def _on_thermostat_changed(self, command_id: str) -> callable:
+        responses = list()
+        # noinspection PyNoneFunctionAssignment
+        temperature = self.fetch_temperature(self._device_id)
+        if temperature is not None:
+            # noinspection PyStringFormat
+            responses.append("temperature:{:02f}".format(temperature))
+
+        # noinspection PyNoneFunctionAssignment
+        thermostat_mode = self.fetch_thermostat_mode(self._device_id)
+        if thermostat_mode:
+            responses.append("thermostatMode:{}".format(thermostat_mode.name))
+
+        # noinspection PyNoneFunctionAssignment
+        thermostat_target_setpoint = self.fetch_thermostat_target_setpoint(self._device_id)
+        if thermostat_target_setpoint is not None:
+            # noinspection PyStringFormat
+            responses.append("thermostatTargetSetpoint:{:02f}".format(thermostat_target_setpoint))
+
+        return lambda: self._command_responded(command_id, *responses)
+
+    def _on_power_status(self, command_id: str) -> callable:
+        responses = list()
+
+        responses.append('ON' if self.fetch_is_power_on(self._device_id) else 'OFF')
+
+        # noinspection PyNoneFunctionAssignment
+        temperature = self.fetch_temperature(self._device_id)
+        if temperature is not None:
+            # noinspection PyStringFormat
+            responses.append("temperature:{:02f}".format(temperature))
+
+        # noinspection PyNoneFunctionAssignment
+        thermostat_mode = self.fetch_thermostat_mode(self._device_id)
+        if thermostat_mode:
+            responses.append("thermostatMode:{}".format(thermostat_mode.name))
+
+        # noinspection PyNoneFunctionAssignment
+        thermostat_target_setpoint = self.fetch_thermostat_target_setpoint(self._device_id)
+        if thermostat_target_setpoint is not None:
+            # noinspection PyStringFormat
+            responses.append("thermostatTargetSetpoint:{:02f}".format(thermostat_target_setpoint))
+
+        return lambda: self._command_responded(command_id, *responses)
+
     def _command_executed(self, command_id: str):
         self._info('Marking command %s as executed', command_id)
         self._mutation(
@@ -197,7 +284,7 @@ class DeviceIdClient:
             }
         )
 
-    def _command_responded(self, command_id: str, response: str):
+    def _command_responded(self, command_id: str, *response: str):
         self._info('Marking command %s as executed', command_id)
         self._mutation(
             '''
@@ -238,9 +325,15 @@ class Client:
     _MINUTES = 60 * _SECONDS
     _HOURS = 60 * _MINUTES
 
-    def __init__(self, configuration: DevicesConfiguration, reconnect_time_seconds: int = 12 * _HOURS):
+    def __init__(
+            self, configuration: DevicesConfiguration, reconnect_time_seconds: int = 12 * _HOURS,
+            dev_environment: bool = False
+    ):
+        appsync_url = _DEV_ROOT_URL if dev_environment else _ROOT_URL
         self._clients_by_device_id = {
-            conf['device_id']: DeviceIdClient(conf['device_id'], conf['openid_token'], reconnect_time_seconds)
+            conf['device_id']: DeviceIdClient(
+                conf['device_id'], conf['openid_token'], reconnect_time_seconds,
+                appsync_url)
             for conf in configuration.get_configuration()['Devices']
         }
         self._logger = logging.getLogger('client.raspi.alexa.mirko.io')
@@ -251,9 +344,27 @@ class Client:
     def set_on_turn_off(self, device_id: str, fun: callable):
         self._clients_by_device_id[device_id].on_turn_off = fun
 
+    def set_fetch_temperature(self, device_id: str, fun: callable):
+        self._clients_by_device_id[device_id].fetch_temperature = fun
+
+    def set_fetch_thermostat_mode(self, device_id: str, fun: callable):
+        self._clients_by_device_id[device_id].fetch_thermostat_mode = fun
+
+    def set_fetch_thermostat_target_setpoint(self, device_id: str, fun: callable):
+        self._clients_by_device_id[device_id].fetch_thermostat_target_setpoint = fun
+
     def set_fetch_is_power_on(self, device_id: str, fun: callable):
         self._clients_by_device_id[device_id].fetch_is_power_on = fun
 
+    def set_on_set_temperature(self, device_id, fun: callable):
+        self._clients_by_device_id[device_id].on_set_temperature = fun
+
+    def set_on_adjust_temperature(self, device_id, fun: callable):
+        self._clients_by_device_id[device_id].on_adjust_temperature = fun
+
+    def set_on_set_thermostat_mode(self, device_id, fun: callable):
+        self._clients_by_device_id[device_id].on_set_thermostat_mode = fun
+
     def listen(self):
         executor = ThreadPoolExecutor(max_workers=len(self._clients_by_device_id))
         # noinspection PyTypeChecker
